@@ -1,44 +1,39 @@
 # -*- coding: utf-8 -*-
+"""Logging configuration.
+
+The Celery instances logging section: ``Celery.log``.
+
+Sets up logging for the worker and other programs,
+redirects standard outs, colors log output, patches logging
+related compatibility fixes, and so on.
 """
-    celery.app.log
-    ~~~~~~~~~~~~~~
-
-    The Celery instances logging section: ``Celery.log``.
-
-    Sets up logging for the worker and other programs,
-    redirects stdouts, colors log output, patches logging
-    related compatibility fixes, and so on.
-
-"""
-from __future__ import absolute_import
+from __future__ import absolute_import, unicode_literals
 
 import logging
 import os
 import sys
-
 from logging.handlers import WatchedFileHandler
 
-from kombu.log import NullHandler
 from kombu.utils.encoding import set_default_encoding_file
 
 from celery import signals
 from celery._state import get_current_task
-from celery.five import class_property, string_t
-from celery.utils import isatty, node_format
-from celery.utils.log import (
-    get_logger, mlevel,
-    ColorFormatter, ensure_process_aware_logger,
-    LoggingProxy, get_multiprocessing_logger,
-    reset_multiprocessing_logger,
-)
+from celery.five import string_t
+from celery.local import class_property
+from celery.platforms import isatty
+from celery.utils.log import (ColorFormatter, LoggingProxy, get_logger,
+                              get_multiprocessing_logger, mlevel,
+                              reset_multiprocessing_logger)
+from celery.utils.nodenames import node_format
 from celery.utils.term import colored
 
-__all__ = ['TaskFormatter', 'Logging']
+__all__ = ('TaskFormatter', 'Logging')
 
 MP_LOG = os.environ.get('MP_LOG', False)
 
 
 class TaskFormatter(ColorFormatter):
+    """Formatter for tasks, adding the task name and id."""
 
     def format(self, record):
         task = get_current_task()
@@ -52,6 +47,8 @@ class TaskFormatter(ColorFormatter):
 
 
 class Logging(object):
+    """Application logging setup (app.log)."""
+
     #: The logging subsystem is only configured once per process.
     #: setup_logging_subsystem sets this flag, and subsequent calls
     #: will do nothing.
@@ -59,13 +56,14 @@ class Logging(object):
 
     def __init__(self, app):
         self.app = app
-        self.loglevel = mlevel(self.app.conf.CELERYD_LOG_LEVEL)
-        self.format = self.app.conf.CELERYD_LOG_FORMAT
-        self.task_format = self.app.conf.CELERYD_TASK_LOG_FORMAT
-        self.colorize = self.app.conf.CELERYD_LOG_COLOR
+        self.loglevel = mlevel(logging.WARN)
+        self.format = self.app.conf.worker_log_format
+        self.task_format = self.app.conf.worker_task_log_format
+        self.colorize = self.app.conf.worker_log_color
 
     def setup(self, loglevel=None, logfile=None, redirect_stdouts=False,
               redirect_level='WARNING', colorize=None, hostname=None):
+        loglevel = mlevel(loglevel)
         handled = self.setup_logging_subsystem(
             loglevel, logfile, colorize=colorize, hostname=hostname,
         )
@@ -93,12 +91,11 @@ class Logging(object):
             return
         if logfile and hostname:
             logfile = node_format(logfile, hostname)
-        self.already_setup = True
+        Logging._setup = True
         loglevel = mlevel(loglevel or self.loglevel)
         format = format or self.format
         colorize = self.supports_color(colorize, logfile)
         reset_multiprocessing_logger()
-        ensure_process_aware_logger()
         receivers = signals.setup_logging.send(
             sender=None, loglevel=loglevel, logfile=logfile,
             format=format, colorize=colorize,
@@ -107,7 +104,7 @@ class Logging(object):
         if not receivers:
             root = logging.getLogger()
 
-            if self.app.conf.CELERYD_HIJACK_ROOT_LOGGER:
+            if self.app.conf.worker_hijack_root_logger:
                 root.handlers = []
                 get_logger('celery').handlers = []
                 get_logger('celery.task').handlers = []
@@ -164,7 +161,6 @@ class Logging(object):
         If `logfile` is not specified, then `sys.stderr` is used.
 
         Will return the base task logger object.
-
         """
         loglevel = mlevel(loglevel or self.loglevel)
         format = format or self.task_format
@@ -176,8 +172,8 @@ class Logging(object):
             formatter=TaskFormatter, **kwargs
         )
         logger.setLevel(loglevel)
-        logger.propagate = int(propagate)    # this is an int for some reason.
-                                             # better to not question why.
+        # this is an int for some reason, better to not question why.
+        logger.propagate = int(propagate)
         signals.after_setup_task_logger.send(
             sender=None, logger=logger,
             loglevel=loglevel, logfile=logfile,
@@ -187,12 +183,12 @@ class Logging(object):
 
     def redirect_stdouts_to_logger(self, logger, loglevel=None,
                                    stdout=True, stderr=True):
-        """Redirect :class:`sys.stdout` and :class:`sys.stderr` to a
-        logging instance.
-
-        :param logger: The :class:`logging.Logger` instance to redirect to.
-        :param loglevel: The loglevel redirected messages will be logged as.
+        """Redirect :class:`sys.stdout` and :class:`sys.stderr` to logger.
 
+        Arguments:
+            logger (logging.Logger): Logger instance to redirect to.
+            loglevel (int, str): The loglevel redirected message
+                will be logged as.
         """
         proxy = LoggingProxy(logger, loglevel)
         if stdout:
@@ -207,7 +203,7 @@ class Logging(object):
             # Windows does not support ANSI color codes.
             return False
         if colorize or colorize is None:
-            # Only use color if there is no active log file
+            # Only use color if there's no active log file
             # and stderr is an actual terminal.
             return logfile is None and isatty(sys.stderr)
         return colorize
@@ -225,16 +221,17 @@ class Logging(object):
         return logger
 
     def _detect_handler(self, logfile=None):
-        """Create log handler with either a filename, an open stream
-        or :const:`None` (stderr)."""
+        """Create handler from filename, an open stream or `None` (stderr)."""
         logfile = sys.__stderr__ if logfile is None else logfile
         if hasattr(logfile, 'write'):
             return logging.StreamHandler(logfile)
         return WatchedFileHandler(logfile)
 
     def _has_handler(self, logger):
-        if logger.handlers:
-            return any(not isinstance(h, NullHandler) for h in logger.handlers)
+        return any(
+            not isinstance(h, logging.NullHandler)
+            for h in logger.handlers or []
+        )
 
     def _is_configured(self, logger):
         return self._has_handler(logger) and not getattr(
@@ -249,9 +246,9 @@ class Logging(object):
         return get_logger(name)
 
     @class_property
-    def already_setup(cls):
-        return cls._setup
+    def already_setup(self):
+        return self._setup
 
     @already_setup.setter  # noqa
-    def already_setup(cls, was_setup):
-        cls._setup = was_setup
+    def already_setup(self, was_setup):
+        self._setup = was_setup
