@@ -1,25 +1,20 @@
 # -*- coding: utf-8 -*-
-"""
-    celery.app.registry
-    ~~~~~~~~~~~~~~~~~~~
-
-    Registry of available tasks.
-
-"""
-from __future__ import absolute_import
+"""Registry of available tasks."""
+from __future__ import absolute_import, unicode_literals
 
 import inspect
-
 from importlib import import_module
 
 from celery._state import get_current_app
-from celery.exceptions import NotRegistered
+from celery.exceptions import InvalidTaskError, NotRegistered
 from celery.five import items
 
-__all__ = ['TaskRegistry']
+__all__ = ('TaskRegistry',)
 
 
 class TaskRegistry(dict):
+    """Map of registered tasks."""
+
     NotRegistered = NotRegistered
 
     def __missing__(self, key):
@@ -29,20 +24,23 @@ class TaskRegistry(dict):
         """Register a task in the task registry.
 
         The task will be automatically instantiated if not already an
-        instance.
-
+        instance. Name must be configured prior to registration.
         """
+        if task.name is None:
+            raise InvalidTaskError(
+                'Task class {0!r} must specify .name attribute'.format(
+                    type(task).__name__))
         self[task.name] = inspect.isclass(task) and task() or task
 
     def unregister(self, name):
         """Unregister task by name.
 
-        :param name: name of the task to unregister, or a
-            :class:`celery.task.base.Task` with a valid `name` attribute.
-
-        :raises celery.exceptions.NotRegistered: if the task has not
-            been registered.
+        Arguments:
+            name (str): name of the task to unregister, or a
+                :class:`celery.task.base.Task` with a valid `name` attribute.
 
+        Raises:
+            celery.exceptions.NotRegistered: if the task is not registered.
         """
         try:
             self.pop(getattr(name, 'name', name))
@@ -57,8 +55,8 @@ class TaskRegistry(dict):
         return self.filter_types('periodic')
 
     def filter_types(self, type):
-        return dict((name, task) for name, task in items(self)
-                    if getattr(task, 'type', 'regular') == type)
+        return {name: task for name, task in items(self)
+                if getattr(task, 'type', 'regular') == type}
 
 
 def _unpickle_task(name):
