@@ -1,38 +1,30 @@
 # -*- coding: utf-8 -*-
-"""
-    celery.schedules
-    ~~~~~~~~~~~~~~~~
-
-    Schedules define the intervals at which periodic tasks
-    should run.
-
-"""
-from __future__ import absolute_import
+"""Schedules define the intervals at which periodic tasks run."""
+from __future__ import absolute_import, unicode_literals
 
+import numbers
 import re
-
-from collections import namedtuple
+from bisect import bisect, bisect_left
+from collections import Iterable, namedtuple
 from datetime import datetime, timedelta
 
-from kombu.utils import cached_property
+from kombu.utils.objects import cached_property
 
 from . import current_app
-from .five import range, string_t
-from .utils import is_iterable
-from .utils.timeutils import (
-    timedelta_seconds, weekday, maybe_timedelta, remaining,
-    humanize_seconds, timezone, maybe_make_aware, ffwd
-)
-from .datastructures import AttributeDict
+from .five import python_2_unicode_compatible, range, string_t
+from .utils.collections import AttributeDict
+from .utils.time import (ffwd, humanize_seconds, localize, maybe_make_aware,
+                         maybe_timedelta, remaining, timezone, weekday)
 
-__all__ = ['ParseException', 'schedule', 'crontab', 'crontab_parser',
-           'maybe_schedule']
+__all__ = (
+    'ParseException', 'schedule', 'crontab', 'crontab_parser',
+    'maybe_schedule', 'solar',
+)
 
 schedstate = namedtuple('schedstate', ('is_due', 'next'))
 
-
 CRON_PATTERN_INVALID = """\
-Invalid crontab pattern. Valid range is {min}-{max}. \
+Invalid crontab pattern.  Valid range is {min}-{max}. \
 '{value}' was found.\
 """
 
@@ -46,37 +38,92 @@ CRON_REPR = """\
 {0._orig_day_of_month} {0._orig_month_of_year} (m/h/d/dM/MY)>\
 """
 
+SOLAR_INVALID_LATITUDE = """\
+Argument latitude {lat} is invalid, must be between -90 and 90.\
+"""
+
+SOLAR_INVALID_LONGITUDE = """\
+Argument longitude {lon} is invalid, must be between -180 and 180.\
+"""
+
+SOLAR_INVALID_EVENT = """\
+Argument event "{event}" is invalid, must be one of {all_events}.\
+"""
+
 
 def cronfield(s):
     return '*' if s is None else s
 
 
 class ParseException(Exception):
-    """Raised by crontab_parser when the input can't be parsed."""
+    """Raised by :class:`crontab_parser` when the input can't be parsed."""
 
 
-class schedule(object):
-    """Schedule for periodic task.
+class BaseSchedule(object):
 
-    :param run_every: Interval in seconds (or a :class:`~datetime.timedelta`).
-    :param relative:  If set to True the run time will be rounded to the
-        resolution of the interval.
-    :param nowfun: Function returning the current date and time
-        (class:`~datetime.datetime`).
-    :param app: Celery app instance.
-
-    """
-    relative = False
-
-    def __init__(self, run_every=None, relative=False, nowfun=None, app=None):
-        self.run_every = maybe_timedelta(run_every)
-        self.relative = relative
+    def __init__(self, nowfun=None, app=None):
         self.nowfun = nowfun
         self._app = app
 
     def now(self):
         return (self.nowfun or self.app.now)()
 
+    def remaining_estimate(self, last_run_at):
+        raise NotImplementedError()
+
+    def is_due(self, last_run_at):
+        raise NotImplementedError()
+
+    def maybe_make_aware(self, dt):
+        return maybe_make_aware(dt, self.tz)
+
+    @property
+    def app(self):
+        return self._app or current_app._get_current_object()
+
+    @app.setter  # noqa
+    def app(self, app):
+        self._app = app
+
+    @cached_property
+    def tz(self):
+        return self.app.timezone
+
+    @cached_property
+    def utc_enabled(self):
+        return self.app.conf.enable_utc
+
+    def to_local(self, dt):
+        if not self.utc_enabled:
+            return timezone.to_local_fallback(dt)
+        return dt
+
+    def __eq__(self, other):
+        if isinstance(other, BaseSchedule):
+            return other.nowfun == self.nowfun
+        return NotImplemented
+
+
+@python_2_unicode_compatible
+class schedule(BaseSchedule):
+    """Schedule for periodic task.
+
+    Arguments:
+        run_every (float, ~datetime.timedelta): Time interval.
+        relative (bool):  If set to True the run time will be rounded to the
+            resolution of the interval.
+        nowfun (Callable): Function returning the current date and time
+            (:class:`~datetime.datetime`).
+        app (Celery): Celery app instance.
+    """
+
+    relative = False
+
+    def __init__(self, run_every=None, relative=False, nowfun=None, app=None):
+        self.run_every = maybe_timedelta(run_every)
+        self.relative = relative
+        super(schedule, self).__init__(nowfun=nowfun, app=app)
+
     def remaining_estimate(self, last_run_at):
         return remaining(
             self.maybe_make_aware(last_run_at), self.run_every,
@@ -84,47 +131,42 @@ class schedule(object):
         )
 
     def is_due(self, last_run_at):
-        """Returns tuple of two items `(is_due, next_time_to_check)`,
-        where next time to check is in seconds.
+        """Return tuple of ``(is_due, next_time_to_check)``.
 
-        e.g.
+        Notes:
+            - next time to check is in seconds.
 
-        * `(True, 20)`, means the task should be run now, and the next
-            time to check is in 20 seconds.
+            - ``(True, 20)``, means the task should be run now, and the next
+                time to check is in 20 seconds.
 
-        * `(False, 12.3)`, means the task is not due, but that the scheduler
-          should check again in 12.3 seconds.
+            - ``(False, 12.3)``, means the task is not due, but that the
+              scheduler should check again in 12.3 seconds.
 
-        The next time to check is used to save energy/cpu cycles,
+        The next time to check is used to save energy/CPU cycles,
         it does not need to be accurate but will influence the precision
         of your schedule.  You must also keep in mind
-        the value of :setting:`CELERYBEAT_MAX_LOOP_INTERVAL`,
-        which decides the maximum number of seconds the scheduler can
+        the value of :setting:`beat_max_loop_interval`,
+        that decides the maximum number of seconds the scheduler can
         sleep between re-checking the periodic task intervals.  So if you
-        have a task that changes schedule at runtime then your next_run_at
+        have a task that changes schedule at run-time then your next_run_at
         check will decide how long it will take before a change to the
-        schedule takes effect.  The max loop interval takes precendence
+        schedule takes effect.  The max loop interval takes precedence
         over the next check at value returned.
 
         .. admonition:: Scheduler max interval variance
 
             The default max loop interval may vary for different schedulers.
-            For the default scheduler the value is 5 minutes, but for e.g.
-            the django-celery database scheduler the value is 5 seconds.
-
+            For the default scheduler the value is 5 minutes, but for example
+            the :pypi:`django-celery-beat` database scheduler the value
+            is 5 seconds.
         """
         last_run_at = self.maybe_make_aware(last_run_at)
         rem_delta = self.remaining_estimate(last_run_at)
-        remaining_s = timedelta_seconds(rem_delta)
+        remaining_s = max(rem_delta.total_seconds(), 0)
         if remaining_s == 0:
             return schedstate(is_due=True, next=self.seconds)
         return schedstate(is_due=False, next=remaining_s)
 
-    def maybe_make_aware(self, dt):
-        if self.utc_enabled:
-            return maybe_make_aware(dt, self.tz)
-        return dt
-
     def __repr__(self):
         return '<freq: {0.human_seconds}>'.format(self)
 
@@ -141,39 +183,22 @@ class schedule(object):
 
     @property
     def seconds(self):
-        return timedelta_seconds(self.run_every)
+        return max(self.run_every.total_seconds(), 0)
 
     @property
     def human_seconds(self):
         return humanize_seconds(self.seconds)
 
-    @property
-    def app(self):
-        return self._app or current_app._get_current_object()
-
-    @app.setter  # noqa
-    def app(self, app):
-        self._app = app
-
-    @cached_property
-    def tz(self):
-        return self.app.timezone
-
-    @cached_property
-    def utc_enabled(self):
-        return self.app.conf.CELERY_ENABLE_UTC
-
-    def to_local(self, dt):
-        if not self.utc_enabled:
-            return timezone.to_local_fallback(dt)
-        return dt
-
 
 class crontab_parser(object):
-    """Parser for crontab expressions. Any expression of the form 'groups'
+    """Parser for Crontab expressions.
+
+    Any expression of the form 'groups'
     (see BNF grammar below) is accepted and expanded to a set of numbers.
-    These numbers represent the units of time that the crontab needs to
-    run on::
+    These numbers represent the units of time that the Crontab needs to
+    run on:
+
+    .. code-block:: bnf
 
         digit   :: '0'..'9'
         dow     :: 'a'..'z'
@@ -185,7 +210,9 @@ class crontab_parser(object):
         groups  :: expr ( ',' expr ) *
 
     The parser is a general purpose one, useful for parsing hours, minutes and
-    day_of_week expressions.  Example usage::
+    day of week expressions.  Example usage:
+
+    .. code-block:: pycon
 
         >>> minutes = crontab_parser(60).parse('*/15')
         [0, 15, 30, 45]
@@ -194,8 +221,10 @@ class crontab_parser(object):
         >>> day_of_week = crontab_parser(7).parse('*')
         [0, 1, 2, 3, 4, 5, 6]
 
-    It can also parse day_of_month and month_of_year expressions if initialized
-    with an minimum of 1.  Example usage::
+    It can also parse day of month and month of year expressions if initialized
+    with a minimum of 1.  Example usage:
+
+    .. code-block:: pycon
 
         >>> days_of_month = crontab_parser(31, 1).parse('*/3')
         [1, 4, 7, 10, 13, 16, 19, 22, 25, 28, 31]
@@ -204,11 +233,11 @@ class crontab_parser(object):
         >>> months_of_year = crontab_parser(12, 1).parse('2-12/2')
         [2, 4, 6, 8, 10, 12]
 
-    The maximum possible expanded value returned is found by the formula::
-
-        max_ + min_ - 1
+    The maximum possible expanded value returned is found by the formula:
 
+        :math:`max_ + min_ - 1`
     """
+
     ParseException = ParseException
 
     _range = r'(\w+?)-(\w+)'
@@ -238,7 +267,7 @@ class crontab_parser(object):
             m = regex.match(part)
             if m:
                 return handler(m.groups())
-        return self._expand_range((part, ))
+        return self._expand_range((part,))
 
     def _expand_range(self, toks):
         fr = self._expand_number(toks[0])
@@ -285,13 +314,16 @@ class crontab_parser(object):
         return i
 
 
-class crontab(schedule):
-    """A crontab can be used as the `run_every` value of a
-    :class:`PeriodicTask` to add cron-like scheduling.
+@python_2_unicode_compatible
+class crontab(BaseSchedule):
+    """Crontab schedule.
 
-    Like a :manpage:`cron` job, you can specify units of time of when
-    you would like the task to execute. It is a reasonably complete
-    implementation of cron's features, so it should provide a fair
+    A Crontab can be used as the ``run_every`` value of a
+    periodic task entry to add :manpage:`crontab(5)`-like scheduling.
+
+    Like a :manpage:`cron(5)`-job, you can specify units of time of when
+    you'd like the task to execute.  It's a reasonably complete
+    implementation of :command:`cron`'s features, so it should provide a fair
     degree of scheduling needs.
 
     You can specify a minute, an hour, a day of the week, a day of the
@@ -301,17 +333,17 @@ class crontab(schedule):
 
         - A (list of) integers from 0-59 that represent the minutes of
           an hour of when execution should occur; or
-        - A string representing a crontab pattern.  This may get pretty
-          advanced, like `minute='*/15'` (for every quarter) or
-          `minute='1,13,30-45,50-59/2'`.
+        - A string representing a Crontab pattern.  This may get pretty
+          advanced, like ``minute='*/15'`` (for every quarter) or
+          ``minute='1,13,30-45,50-59/2'``.
 
     .. attribute:: hour
 
         - A (list of) integers from 0-23 that represent the hours of
           a day of when execution should occur; or
-        - A string representing a crontab pattern.  This may get pretty
-          advanced, like `hour='*/3'` (for every three hours) or
-          `hour='0,8-17/2'` (at midnight, and every two hours during
+        - A string representing a Crontab pattern.  This may get pretty
+          advanced, like ``hour='*/3'`` (for every three hours) or
+          ``hour='0,8-17/2'`` (at midnight, and every two hours during
           office hours).
 
     .. attribute:: day_of_week
@@ -319,27 +351,27 @@ class crontab(schedule):
         - A (list of) integers from 0-6, where Sunday = 0 and Saturday =
           6, that represent the days of a week that execution should
           occur.
-        - A string representing a crontab pattern.  This may get pretty
-          advanced, like `day_of_week='mon-fri'` (for weekdays only).
-          (Beware that `day_of_week='*/2'` does not literally mean
+        - A string representing a Crontab pattern.  This may get pretty
+          advanced, like ``day_of_week='mon-fri'`` (for weekdays only).
+          (Beware that ``day_of_week='*/2'`` does not literally mean
           'every two days', but 'every day that is divisible by two'!)
 
     .. attribute:: day_of_month
 
         - A (list of) integers from 1-31 that represents the days of the
           month that execution should occur.
-        - A string representing a crontab pattern.  This may get pretty
-          advanced, such as `day_of_month='2-30/3'` (for every even
-          numbered day) or `day_of_month='1-7,15-21'` (for the first and
+        - A string representing a Crontab pattern.  This may get pretty
+          advanced, such as ``day_of_month='2-30/2'`` (for every even
+          numbered day) or ``day_of_month='1-7,15-21'`` (for the first and
           third weeks of the month).
 
     .. attribute:: month_of_year
 
         - A (list of) integers from 1-12 that represents the months of
           the year during which execution can occur.
-        - A string representing a crontab pattern.  This may get pretty
-          advanced, such as `month_of_year='*/3'` (for the first month
-          of every quarter) or `month_of_year='2-12/2'` (for every even
+        - A string representing a Crontab pattern.  This may get pretty
+          advanced, such as ``month_of_year='*/3'`` (for the first month
+          of every quarter) or ``month_of_year='2-12/2'`` (for every even
           numbered month).
 
     .. attribute:: nowfun
@@ -351,63 +383,65 @@ class crontab(schedule):
 
         The Celery app instance.
 
-    It is important to realize that any day on which execution should
+    It's important to realize that any day on which execution should
     occur must be represented by entries in all three of the day and
-    month attributes.  For example, if `day_of_week` is 0 and `day_of_month`
-    is every seventh day, only months that begin on Sunday and are also
-    in the `month_of_year` attribute will have execution events.  Or,
-    `day_of_week` is 1 and `day_of_month` is '1-7,15-21' means every
-    first and third monday of every month present in `month_of_year`.
-
+    month attributes.  For example, if ``day_of_week`` is 0 and
+    ``day_of_month`` is every seventh day, only months that begin
+    on Sunday and are also in the ``month_of_year`` attribute will have
+    execution events.  Or, ``day_of_week`` is 1 and ``day_of_month``
+    is '1-7,15-21' means every first and third Monday of every month
+    present in ``month_of_year``.
     """
 
     def __init__(self, minute='*', hour='*', day_of_week='*',
-                 day_of_month='*', month_of_year='*', nowfun=None, app=None):
+                 day_of_month='*', month_of_year='*', **kwargs):
         self._orig_minute = cronfield(minute)
         self._orig_hour = cronfield(hour)
         self._orig_day_of_week = cronfield(day_of_week)
         self._orig_day_of_month = cronfield(day_of_month)
         self._orig_month_of_year = cronfield(month_of_year)
+        self._orig_kwargs = kwargs
         self.hour = self._expand_cronspec(hour, 24)
         self.minute = self._expand_cronspec(minute, 60)
         self.day_of_week = self._expand_cronspec(day_of_week, 7)
         self.day_of_month = self._expand_cronspec(day_of_month, 31, 1)
         self.month_of_year = self._expand_cronspec(month_of_year, 12, 1)
-        self.nowfun = nowfun
-        self._app = app
+        super(crontab, self).__init__(**kwargs)
 
     @staticmethod
     def _expand_cronspec(cronspec, max_, min_=0):
-        """Takes the given cronspec argument in one of the forms::
+        """Expand cron specification.
+
+        Takes the given cronspec argument in one of the forms:
+
+        .. code-block:: text
 
             int         (like 7)
             str         (like '3-5,*/15', '*', or 'monday')
-            set         (like set([0,15,30,45]))
+            set         (like {0,15,30,45}
             list        (like [8-17])
 
         And convert it to an (expanded) set representing all time unit
-        values on which the crontab triggers.  Only in case of the base
-        type being 'str', parsing occurs.  (It is fast and
-        happens only once for each crontab instance, so there is no
+        values on which the Crontab triggers.  Only in case of the base
+        type being :class:`str`, parsing occurs.  (It's fast and
+        happens only once for each Crontab instance, so there's no
         significant performance overhead involved.)
 
         For the other base types, merely Python type conversions happen.
 
-        The argument `max_` is needed to determine the expansion of '*'
-        and ranges.
-        The argument `min_` is needed to determine the expansion of '*'
-        and ranges for 1-based cronspecs, such as day of month or month
-        of year. The default is sufficient for minute, hour, and day of
-        week.
-
+        The argument ``max_`` is needed to determine the expansion of
+        ``*`` and ranges.  The argument ``min_`` is needed to determine
+        the expansion of ``*`` and ranges for 1-based cronspecs, such as
+        day of month or month of year.  The default is sufficient for minute,
+        hour, and day of week.
         """
-        if isinstance(cronspec, int):
-            result = set([cronspec])
+        if isinstance(cronspec, numbers.Integral):
+            result = {cronspec}
         elif isinstance(cronspec, string_t):
             result = crontab_parser(max_, min_).parse(cronspec)
         elif isinstance(cronspec, set):
             result = cronspec
-        elif is_iterable(cronspec):
+        elif isinstance(cronspec, Iterable):
             result = set(cronspec)
         else:
             raise TypeError(CRON_INVALID_TYPE.format(type=type(cronspec)))
@@ -420,14 +454,15 @@ class crontab(schedule):
         return result
 
     def _delta_to_next(self, last_run_at, next_hour, next_minute):
-        """
-        Takes a datetime of last run, next minute and hour, and
-        returns a relativedelta for the next scheduled day and time.
-        Only called when day_of_month and/or month_of_year cronspec
-        is specified to further limit scheduled task execution.
-        """
-        from bisect import bisect, bisect_left
+        """Find next delta.
 
+        Takes a :class:`~datetime.datetime` of last run, next minute and hour,
+        and returns a :class:`~celery.utils.time.ffwd` for the next
+        scheduled day and time.
+
+        Only called when ``day_of_month`` and/or ``month_of_year``
+        cronspec is specified to further limit scheduled task execution.
+        """
         datedata = AttributeDict(year=last_run_at.year)
         days_of_month = sorted(self.day_of_month)
         months_of_year = sorted(self.month_of_year)
@@ -439,15 +474,20 @@ class crontab(schedule):
                 return True
             return False
 
+        def is_before_last_run(year, month, day):
+            return self.maybe_make_aware(datetime(year,
+                                                  month,
+                                                  day)) < last_run_at
+
         def roll_over():
-            while 1:
+            for _ in range(2000):
                 flag = (datedata.dom == len(days_of_month) or
                         day_out_of_range(datedata.year,
                                          months_of_year[datedata.moy],
                                          days_of_month[datedata.dom]) or
-                        (self.maybe_make_aware(datetime(datedata.year,
-                         months_of_year[datedata.moy],
-                         days_of_month[datedata.dom])) < last_run_at))
+                        (is_before_last_run(datedata.year,
+                                            months_of_year[datedata.moy],
+                                            days_of_month[datedata.dom])))
 
                 if flag:
                     datedata.dom = 0
@@ -457,6 +497,10 @@ class crontab(schedule):
                         datedata.year += 1
                 else:
                     break
+            else:
+                # Tried 2000 times, we're most likely in an infinite loop
+                raise RuntimeError('unable to rollover, '
+                                   'time specification is probably invalid')
 
         if last_run_at.month in self.month_of_year:
             datedata.dom = bisect(days_of_month, last_run_at.day)
@@ -485,9 +529,6 @@ class crontab(schedule):
                     second=0,
                     microsecond=0)
 
-    def now(self):
-        return (self.nowfun or self.app.now)()
-
     def __repr__(self):
         return CRON_REPR.format(self)
 
@@ -496,24 +537,35 @@ class crontab(schedule):
                                  self._orig_hour,
                                  self._orig_day_of_week,
                                  self._orig_day_of_month,
-                                 self._orig_month_of_year), None)
+                                 self._orig_month_of_year), self._orig_kwargs)
+
+    def __setstate__(self, state):
+        # Calling super's init because the kwargs aren't necessarily passed in
+        # the same form as they are stored by the superclass
+        super(crontab, self).__init__(**state)
 
     def remaining_delta(self, last_run_at, tz=None, ffwd=ffwd):
+        # pylint: disable=redefined-outer-name
+        # caching global ffwd
         tz = tz or self.tz
         last_run_at = self.maybe_make_aware(last_run_at)
         now = self.maybe_make_aware(self.now())
         dow_num = last_run_at.isoweekday() % 7  # Sunday is day 0, not day 7
 
-        execute_this_date = (last_run_at.month in self.month_of_year and
-                             last_run_at.day in self.day_of_month and
-                             dow_num in self.day_of_week)
+        execute_this_date = (
+            last_run_at.month in self.month_of_year and
+            last_run_at.day in self.day_of_month and
+            dow_num in self.day_of_week
+        )
 
-        execute_this_hour = (execute_this_date and
-                             last_run_at.day == now.day and
-                             last_run_at.month == now.month and
-                             last_run_at.year == now.year and
-                             last_run_at.hour in self.hour and
-                             last_run_at.minute < max(self.minute))
+        execute_this_hour = (
+            execute_this_date and
+            last_run_at.day == now.day and
+            last_run_at.month == now.month and
+            last_run_at.year == now.year and
+            last_run_at.hour in self.hour and
+            last_run_at.minute < max(self.minute)
+        )
 
         if execute_this_hour:
             next_minute = min(minute for minute in self.minute
@@ -538,55 +590,251 @@ class crontab(schedule):
                                     if day > dow_num] or self.day_of_week)
                     add_week = next_day == dow_num
 
-                    delta = ffwd(weeks=add_week and 1 or 0,
-                                 weekday=(next_day - 1) % 7,
-                                 hour=next_hour,
-                                 minute=next_minute,
-                                 second=0,
-                                 microsecond=0)
+                    delta = ffwd(
+                        weeks=add_week and 1 or 0,
+                        weekday=(next_day - 1) % 7,
+                        hour=next_hour,
+                        minute=next_minute,
+                        second=0,
+                        microsecond=0,
+                    )
                 else:
                     delta = self._delta_to_next(last_run_at,
                                                 next_hour, next_minute)
         return self.to_local(last_run_at), delta, self.to_local(now)
 
     def remaining_estimate(self, last_run_at, ffwd=ffwd):
-        """Returns when the periodic task should run next as a timedelta."""
+        """Estimate of next run time.
+
+        Returns when the periodic task should run next as a
+        :class:`~datetime.timedelta`.
+        """
+        # pylint: disable=redefined-outer-name
+        # caching global ffwd
         return remaining(*self.remaining_delta(last_run_at, ffwd=ffwd))
 
     def is_due(self, last_run_at):
-        """Returns tuple of two items `(is_due, next_time_to_run)`,
-        where next time to run is in seconds.
+        """Return tuple of ``(is_due, next_time_to_run)``.
 
-        See :meth:`celery.schedules.schedule.is_due` for more information.
+        Note:
+            Next time to run is in seconds.
 
+        SeeAlso:
+            :meth:`celery.schedules.schedule.is_due` for more information.
         """
         rem_delta = self.remaining_estimate(last_run_at)
-        rem = timedelta_seconds(rem_delta)
+        rem = max(rem_delta.total_seconds(), 0)
         due = rem == 0
         if due:
             rem_delta = self.remaining_estimate(self.now())
-            rem = timedelta_seconds(rem_delta)
+            rem = max(rem_delta.total_seconds(), 0)
         return schedstate(due, rem)
 
     def __eq__(self, other):
         if isinstance(other, crontab):
-            return (other.month_of_year == self.month_of_year and
-                    other.day_of_month == self.day_of_month and
-                    other.day_of_week == self.day_of_week and
-                    other.hour == self.hour and
-                    other.minute == self.minute)
+            return (
+                other.month_of_year == self.month_of_year and
+                other.day_of_month == self.day_of_month and
+                other.day_of_week == self.day_of_week and
+                other.hour == self.hour and
+                other.minute == self.minute and
+                super(crontab, self).__eq__(other)
+            )
         return NotImplemented
 
     def __ne__(self, other):
-        return not self.__eq__(other)
+        res = self.__eq__(other)
+        if res is NotImplemented:
+            return True
+        return not res
 
 
 def maybe_schedule(s, relative=False, app=None):
+    """Return schedule from number, timedelta, or actual schedule."""
     if s is not None:
-        if isinstance(s, int):
+        if isinstance(s, numbers.Number):
             s = timedelta(seconds=s)
         if isinstance(s, timedelta):
             return schedule(s, relative, app=app)
         else:
             s.app = app
     return s
+
+
+@python_2_unicode_compatible
+class solar(BaseSchedule):
+    """Solar event.
+
+    A solar event can be used as the ``run_every`` value of a
+    periodic task entry to schedule based on certain solar events.
+
+    Notes:
+
+        Available event valus are:
+
+            - ``dawn_astronomical``
+            - ``dawn_nautical``
+            - ``dawn_civil``
+            - ``sunrise``
+            - ``solar_noon``
+            - ``sunset``
+            - ``dusk_civil``
+            - ``dusk_nautical``
+            - ``dusk_astronomical``
+
+    Arguments:
+        event (str): Solar event that triggers this task.
+            See note for available values.
+        lat (int): The latitude of the observer.
+        lon (int): The longitude of the observer.
+        nowfun (Callable): Function returning the current date and time
+            as a class:`~datetime.datetime`.
+        app (Celery): Celery app instance.
+    """
+
+    _all_events = {
+        'dawn_astronomical',
+        'dawn_nautical',
+        'dawn_civil',
+        'sunrise',
+        'solar_noon',
+        'sunset',
+        'dusk_civil',
+        'dusk_nautical',
+        'dusk_astronomical',
+    }
+    _horizons = {
+        'dawn_astronomical': '-18',
+        'dawn_nautical': '-12',
+        'dawn_civil': '-6',
+        'sunrise': '-0:34',
+        'solar_noon': '0',
+        'sunset': '-0:34',
+        'dusk_civil': '-6',
+        'dusk_nautical': '-12',
+        'dusk_astronomical': '18',
+    }
+    _methods = {
+        'dawn_astronomical': 'next_rising',
+        'dawn_nautical': 'next_rising',
+        'dawn_civil': 'next_rising',
+        'sunrise': 'next_rising',
+        'solar_noon': 'next_transit',
+        'sunset': 'next_setting',
+        'dusk_civil': 'next_setting',
+        'dusk_nautical': 'next_setting',
+        'dusk_astronomical': 'next_setting',
+    }
+    _use_center_l = {
+        'dawn_astronomical': True,
+        'dawn_nautical': True,
+        'dawn_civil': True,
+        'sunrise': False,
+        'solar_noon': False,
+        'sunset': False,
+        'dusk_civil': True,
+        'dusk_nautical': True,
+        'dusk_astronomical': True,
+    }
+
+    def __init__(self, event, lat, lon, **kwargs):
+        self.ephem = __import__('ephem')
+        self.event = event
+        self.lat = lat
+        self.lon = lon
+        super(solar, self).__init__(**kwargs)
+
+        if event not in self._all_events:
+            raise ValueError(SOLAR_INVALID_EVENT.format(
+                event=event, all_events=', '.join(sorted(self._all_events)),
+            ))
+        if lat < -90 or lat > 90:
+            raise ValueError(SOLAR_INVALID_LATITUDE.format(lat=lat))
+        if lon < -180 or lon > 180:
+            raise ValueError(SOLAR_INVALID_LONGITUDE.format(lon=lon))
+
+        cal = self.ephem.Observer()
+        cal.lat = str(lat)
+        cal.lon = str(lon)
+        cal.elev = 0
+        cal.horizon = self._horizons[event]
+        cal.pressure = 0
+        self.cal = cal
+
+        self.method = self._methods[event]
+        self.use_center = self._use_center_l[event]
+
+    def __reduce__(self):
+        return self.__class__, (self.event, self.lat, self.lon)
+
+    def __repr__(self):
+        return '<solar: {0} at latitude {1}, longitude: {2}>'.format(
+            self.event, self.lat, self.lon,
+        )
+
+    def remaining_estimate(self, last_run_at):
+        """Return estimate of next time to run.
+
+        Returns:
+            ~datetime.timedelta: when the periodic task should
+                run next, or if it shouldn't run today (e.g., the sun does
+                not rise today), returns the time when the next check
+                should take place.
+        """
+        last_run_at = self.maybe_make_aware(last_run_at)
+        last_run_at_utc = localize(last_run_at, timezone.utc)
+        self.cal.date = last_run_at_utc
+        try:
+            if self.use_center:
+                next_utc = getattr(self.cal, self.method)(
+                    self.ephem.Sun(),
+                    start=last_run_at_utc, use_center=self.use_center
+                )
+            else:
+                next_utc = getattr(self.cal, self.method)(
+                    self.ephem.Sun(), start=last_run_at_utc
+                )
+
+        except self.ephem.CircumpolarError:  # pragma: no cover
+            # Sun won't rise/set today.  Check again tomorrow
+            # (specifically, after the next anti-transit).
+            next_utc = (
+                self.cal.next_antitransit(self.ephem.Sun()) +
+                timedelta(minutes=1)
+            )
+        next = self.maybe_make_aware(next_utc.datetime())
+        now = self.maybe_make_aware(self.now())
+        delta = next - now
+        return delta
+
+    def is_due(self, last_run_at):
+        """Return tuple of ``(is_due, next_time_to_run)``.
+
+        Note:
+            next time to run is in seconds.
+
+        See Also:
+            :meth:`celery.schedules.schedule.is_due` for more information.
+        """
+        rem_delta = self.remaining_estimate(last_run_at)
+        rem = max(rem_delta.total_seconds(), 0)
+        due = rem == 0
+        if due:
+            rem_delta = self.remaining_estimate(self.now())
+            rem = max(rem_delta.total_seconds(), 0)
+        return schedstate(due, rem)
+
+    def __eq__(self, other):
+        if isinstance(other, solar):
+            return (
+                other.event == self.event and
+                other.lat == self.lat and
+                other.lon == self.lon
+            )
+        return NotImplemented
+
+    def __ne__(self, other):
+        res = self.__eq__(other)
+        if res is NotImplemented:
+            return True
+        return not res
