@@ -1,19 +1,10 @@
 # -*- coding: utf-8 -*-
-"""
-    celery.security
-    ~~~~~~~~~~~~~~~
-
-    Module implementing the signing message serializer.
-
-"""
-from __future__ import absolute_import
-
+"""Message Signing Serializer."""
+from __future__ import absolute_import, unicode_literals
 from kombu.serialization import (
     registry, disable_insecure_serializers as _disable_insecure_serializers,
 )
-
 from celery.exceptions import ImproperlyConfigured
-
 from .serialization import register_auth
 
 SSL_NOT_INSTALLED = """\
@@ -25,15 +16,15 @@ Please install by:
 
 SETTING_MISSING = """\
 Sorry, but you have to configure the
-    * CELERY_SECURITY_KEY
-    * CELERY_SECURITY_CERTIFICATE, and the
-    * CELERY_SECURITY_CERT_STORE
+    * security_key
+    * security_certificate, and the
+    * security_cert_storE
 configuration settings to use the auth serializer.
 
 Please see the configuration reference for more information.
 """
 
-__all__ = ['setup_security']
+__all__ = ('setup_security',)
 
 
 def setup_security(allowed_serializers=None, key=None, cert=None, store=None,
@@ -46,7 +37,7 @@ def setup_security(allowed_serializers=None, key=None, cert=None, store=None,
     _disable_insecure_serializers(allowed_serializers)
 
     conf = app.conf
-    if conf.CELERY_TASK_SERIALIZER != 'auth':
+    if conf.task_serializer != 'auth':
         return
 
     try:
@@ -54,9 +45,9 @@ def setup_security(allowed_serializers=None, key=None, cert=None, store=None,
     except ImportError:
         raise ImproperlyConfigured(SSL_NOT_INSTALLED)
 
-    key = key or conf.CELERY_SECURITY_KEY
-    cert = cert or conf.CELERY_SECURITY_CERTIFICATE
-    store = store or conf.CELERY_SECURITY_CERT_STORE
+    key = key or conf.security_key
+    cert = cert or conf.security_certificate
+    store = store or conf.security_cert_store
 
     if not (key and cert and store):
         raise ImproperlyConfigured(SETTING_MISSING)
