@@ -1,24 +1,19 @@
 # -*- coding: utf-8 -*-
+"""Old deprecated task module.
+
+This is the old task module, it shouldn't be used anymore,
+import from the main 'celery' module instead.
+If you're looking for the decorator implementation then that's in
+``celery.app.base.Celery.task``.
 """
-    celery.task
-    ~~~~~~~~~~~
-
-    This is the old task module, it should not be used anymore,
-    import from the main 'celery' module instead.
-    If you're looking for the decorator implementation then that's in
-    ``celery.app.base.Celery.task``.
-
-"""
-from __future__ import absolute_import
-
+from __future__ import absolute_import, unicode_literals
 from celery._state import current_app, current_task as current
-from celery.five import LazyModule, recreate_module
-from celery.local import Proxy
+from celery.local import LazyModule, Proxy, recreate_module
 
-__all__ = [
+__all__ = (
     'BaseTask', 'Task', 'PeriodicTask', 'task', 'periodic_task',
-    'group', 'chord', 'subtask', 'TaskSet',
-]
+    'group', 'chord', 'subtask',
+)
 
 
 STATICA_HACK = True
@@ -29,7 +24,6 @@ if STATICA_HACK:  # pragma: no cover
     # they contain.
     from celery.canvas import group, chord, subtask
     from .base import BaseTask, Task, PeriodicTask, task, periodic_task
-    from .sets import TaskSet
 
 
 class module(LazyModule):
@@ -44,7 +38,6 @@ old_module, new_module = recreate_module(  # pragma: no cover
         'celery.task.base': ['BaseTask', 'Task', 'PeriodicTask',
                              'task', 'periodic_task'],
         'celery.canvas': ['group', 'chord', 'subtask'],
-        'celery.task.sets': ['TaskSet'],
     },
     base=module,
     __package__='celery.task',
