@@ -1,39 +1,38 @@
 # -*- coding: utf-8 -*-
+"""Internal worker state (global).
+
+This includes the currently active and reserved tasks,
+statistics, and revoked tasks.
 """
-    celery.worker.state
-    ~~~~~~~~~~~~~~~~~~~
-
-    Internal worker state (global)
-
-    This includes the currently active and reserved tasks,
-    statistics, and revoked tasks.
-
-"""
-from __future__ import absolute_import
+from __future__ import absolute_import, print_function, unicode_literals
 
 import os
-import sys
 import platform
 import shelve
+import sys
+import weakref
 import zlib
 
 from kombu.serialization import pickle, pickle_protocol
-from kombu.utils import cached_property
+from kombu.utils.objects import cached_property
 
 from celery import __version__
-from celery.datastructures import LimitedSet
 from celery.exceptions import WorkerShutdown, WorkerTerminate
 from celery.five import Counter
+from celery.utils.collections import LimitedSet
 
-__all__ = ['SOFTWARE_INFO', 'reserved_requests', 'active_requests',
-           'total_count', 'revoked', 'task_reserved', 'maybe_shutdown',
-           'task_accepted', 'task_ready', 'task_reserved', 'task_ready',
-           'Persistent']
+__all__ = (
+    'SOFTWARE_INFO', 'reserved_requests', 'active_requests',
+    'total_count', 'revoked', 'task_reserved', 'maybe_shutdown',
+    'task_accepted', 'task_ready', 'Persistent',
+)
 
 #: Worker software/platform information.
-SOFTWARE_INFO = {'sw_ident': 'py-celery',
-                 'sw_ver': __version__,
-                 'sw_sys': platform.system()}
+SOFTWARE_INFO = {
+    'sw_ident': 'py-celery',
+    'sw_ver': __version__,
+    'sw_sys': platform.system(),
+}
 
 #: maximum number of revokes to keep in memory.
 REVOKES_MAX = 50000
@@ -42,11 +41,14 @@ REVOKES_MAX = 50000
 #: being expired when the max limit has been exceeded.
 REVOKE_EXPIRES = 10800
 
-#: set of all reserved :class:`~celery.worker.job.Request`'s.
-reserved_requests = set()
+#: Mapping of reserved task_id->Request.
+requests = {}
 
-#: set of currently active :class:`~celery.worker.job.Request`'s.
-active_requests = set()
+#: set of all reserved :class:`~celery.worker.request.Request`'s.
+reserved_requests = weakref.WeakSet()
+
+#: set of currently active :class:`~celery.worker.request.Request`'s.
+active_requests = weakref.WeakSet()
 
 #: count of tasks accepted by the worker, sorted by type.
 total_count = Counter()
@@ -54,34 +56,56 @@ total_count = Counter()
 #: count of all tasks accepted by the worker
 all_total_count = [0]
 
-#: the list of currently revoked tasks.  Persistent if statedb set.
+#: the list of currently revoked tasks.  Persistent if ``statedb`` set.
 revoked = LimitedSet(maxlen=REVOKES_MAX, expires=REVOKE_EXPIRES)
 
-#: Update global state when a task has been reserved.
-task_reserved = reserved_requests.add
+should_stop = None
+should_terminate = None
 
-should_stop = False
-should_terminate = False
+
+def reset_state():
+    requests.clear()
+    reserved_requests.clear()
+    active_requests.clear()
+    total_count.clear()
+    all_total_count[:] = [0]
+    revoked.clear()
 
 
 def maybe_shutdown():
-    if should_stop:
-        raise WorkerShutdown()
-    elif should_terminate:
-        raise WorkerTerminate()
+    """Shutdown if flags have been set."""
+    if should_stop is not None and should_stop is not False:
+        raise WorkerShutdown(should_stop)
+    elif should_terminate is not None and should_terminate is not False:
+        raise WorkerTerminate(should_terminate)
 
 
-def task_accepted(request, _all_total_count=all_total_count):
-    """Updates global state when a task has been accepted."""
-    active_requests.add(request)
-    total_count[request.name] += 1
+def task_reserved(request,
+                  add_request=requests.__setitem__,
+                  add_reserved_request=reserved_requests.add):
+    """Update global state when a task has been reserved."""
+    add_request(request.id, request)
+    add_reserved_request(request)
+
+
+def task_accepted(request,
+                  _all_total_count=all_total_count,
+                  add_active_request=active_requests.add,
+                  add_to_total_count=total_count.update):
+    """Update global state when a task has been accepted."""
+    add_active_request(request)
+    add_to_total_count({request.name: 1})
     all_total_count[0] += 1
 
 
-def task_ready(request):
-    """Updates global state when a task is ready."""
-    active_requests.discard(request)
-    reserved_requests.discard(request)
+def task_ready(request,
+               remove_request=requests.pop,
+               discard_active_request=active_requests.discard,
+               discard_reserved_request=reserved_requests.discard):
+    """Update global state when a task is ready."""
+    remove_request(request.id, None)
+    discard_active_request(request)
+    discard_reserved_request(request)
 
 
 C_BENCH = os.environ.get('C_BENCH') or os.environ.get('CELERY_BENCH')
@@ -90,7 +114,7 @@ C_BENCH_EVERY = int(os.environ.get('C_BENCH_EVERY') or
 if C_BENCH:  # pragma: no cover
     import atexit
 
-    from billiard import current_process
+    from billiard.process import current_process
     from celery.five import monotonic
     from celery.utils.debug import memdump, sample_mem
 
@@ -114,6 +138,7 @@ if C_BENCH:  # pragma: no cover
                 memdump()
 
     def task_reserved(request):  # noqa
+        """Called when a task is reserved by the worker."""
         global bench_start
         global bench_first
         now = None
@@ -125,6 +150,7 @@ if C_BENCH:  # pragma: no cover
         return __reserved(request)
 
     def task_ready(request):  # noqa
+        """Called when a task is completed."""
         global all_count
         global bench_start
         global bench_last
@@ -142,12 +168,14 @@ if C_BENCH:  # pragma: no cover
 
 
 class Persistent(object):
-    """This is the persistent data stored by the worker when
-    :option:`--statedb` is enabled.
+    """Stores worker state between restarts.
 
-    It currently only stores revoked task id's.
+    This is the persistent data stored by the worker when
+    :option:`celery worker --statedb` is enabled.
 
+    Currently only stores revoked task id's.
     """
+
     storage = shelve
     protocol = pickle_protocol
     compress = zlib.compress
@@ -188,23 +216,23 @@ class Persistent(object):
 
     def _sync_with(self, d):
         self._revoked_tasks.purge()
-        d.update(
-            __proto__=3,
-            zrevoked=self.compress(self._dumps(self._revoked_tasks)),
-            clock=self.clock.forward() if self.clock else 0,
-        )
+        d.update({
+            str('__proto__'): 3,
+            str('zrevoked'): self.compress(self._dumps(self._revoked_tasks)),
+            str('clock'): self.clock.forward() if self.clock else 0,
+        })
         return d
 
     def _merge_clock(self, d):
         if self.clock:
-            d['clock'] = self.clock.adjust(d.get('clock') or 0)
+            d[str('clock')] = self.clock.adjust(d.get(str('clock')) or 0)
 
     def _merge_revoked(self, d):
         try:
-            self._merge_revoked_v3(d['zrevoked'])
+            self._merge_revoked_v3(d[str('zrevoked')])
         except KeyError:
             try:
-                self._merge_revoked_v2(d.pop('revoked'))
+                self._merge_revoked_v2(d.pop(str('revoked')))
             except KeyError:
                 pass
         # purge expired items at boot
