@@ -13,7 +13,7 @@ globals().update(conf.build_config(
     github_project='celery/celery',
     author='Ask Solem & contributors',
     author_name='Ask Solem',
-    copyright='2009-2016',
+    copyright='2009-2018',
     publisher='Celery Project',
     html_logo='images/celery_512.png',
     html_favicon='images/favicon.ico',
@@ -45,6 +45,9 @@ globals().update(conf.build_config(
         'celery.utils.encoding',
         r'celery.utils.static.*',
     ],
+    linkcheck_ignore=[
+        r'^http://localhost'
+    ]
 ))
 
 settings = {}
