@@ -1,3 +1,5 @@
+# Create your tasks here
+from __future__ import absolute_import, unicode_literals
 from celery import shared_task
 
 
