@@ -1,8 +1,11 @@
 from __future__ import absolute_import, unicode_literals
-import pytest
+
 from datetime import datetime, timedelta
+
+import pytest
 from case import Mock
 from kombu import Exchange, Queue
+
 from celery import uuid
 from celery.app.amqp import Queues, utf8dict
 from celery.five import keys
@@ -135,24 +138,24 @@ class test_Queues:
         assert q['barfoo'] is q['foo']
 
     @pytest.mark.parametrize('queues_kwargs,qname,q,expected', [
-        (dict(max_priority=10),
+        ({'max_priority': 10},
          'foo', 'foo', {'x-max-priority': 10}),
-        (dict(max_priority=10),
+        ({'max_priority': 10},
          'xyz', Queue('xyz', queue_arguments={'x-max-priority': 3}),
          {'x-max-priority': 3}),
-        (dict(max_priority=10),
+        ({'max_priority': 10},
          'moo', Queue('moo', queue_arguments=None),
          {'x-max-priority': 10}),
-        (dict(ha_policy='all', max_priority=5),
+        ({'ha_policy': 'all', 'max_priority': 5},
          'bar', 'bar',
          {'x-ha-policy': 'all', 'x-max-priority': 5}),
-        (dict(ha_policy='all', max_priority=5),
+        ({'ha_policy': 'all', 'max_priority': 5},
          'xyx2', Queue('xyx2', queue_arguments={'x-max-priority': 2}),
          {'x-ha-policy': 'all', 'x-max-priority': 2}),
-        (dict(max_priority=None),
+        ({'max_priority': None},
          'foo2', 'foo2',
          None),
-        (dict(max_priority=None),
+        ({'max_priority': None},
          'xyx3', Queue('xyx3', queue_arguments={'x-max-priority': 7}),
          {'x-max-priority': 7}),
 
@@ -356,6 +359,13 @@ class test_as_task_v2:
         assert m.headers['expires'] == (
             now + timedelta(seconds=30)).isoformat()
 
+    def test_eta_to_datetime(self):
+        eta = datetime.utcnow()
+        m = self.app.amqp.as_task_v2(
+            uuid(), 'foo', eta=eta,
+        )
+        assert m.headers['eta'] == eta.isoformat()
+
     def test_callbacks_errbacks_chord(self):
 
         @self.app.task
