@@ -1,21 +1,19 @@
 from __future__ import absolute_import, unicode_literals
 
-from datetime import datetime, timedelta
 import gc
 import itertools
 import os
-import pytest
-
 from copy import deepcopy
-from pickle import loads, dumps
+from datetime import datetime, timedelta
+from pickle import dumps, loads
 
+import pytest
 from case import ContextMock, Mock, mock, patch
 from vine import promise
 
-from celery import Celery
-from celery import shared_task, current_app
+from celery import Celery, _state
 from celery import app as _app
-from celery import _state
+from celery import current_app, shared_task
 from celery.app import base as _appbase
 from celery.app import defaults
 from celery.exceptions import ImproperlyConfigured
@@ -23,9 +21,9 @@ from celery.five import items, keys
 from celery.loaders.base import unconfigured
 from celery.platforms import pyimplementation
 from celery.utils.collections import DictAttribute
-from celery.utils.serialization import pickle
-from celery.utils.time import timezone, to_utc, localize
 from celery.utils.objects import Bunch
+from celery.utils.serialization import pickle
+from celery.utils.time import localize, timezone, to_utc
 
 THIS_IS_A_KEY = 'this is a value'
 
@@ -36,7 +34,7 @@ class ObjectConfig(object):
 
 
 object_config = ObjectConfig()
-dict_config = dict(FOO=10, BAR=20)
+dict_config = {'FOO': 10, 'BAR': 20}
 
 
 class ObjectConfig2(object):
@@ -79,10 +77,10 @@ class test_App:
         tz_utc = timezone.get_timezone('UTC')
         tz_us_eastern = timezone.get_timezone(timezone_setting_value)
 
-        now = datetime.utcnow().replace(tzinfo=tz_utc)
+        now = to_utc(datetime.utcnow())
         app_now = self.app.now()
 
-        assert app_now.tzinfo == tz_utc
+        assert app_now.tzinfo is tz_utc
         assert app_now - now <= timedelta(seconds=1)
 
         # Check that timezone conversion is applied from configuration
@@ -92,7 +90,8 @@ class test_App:
         del self.app.timezone
 
         app_now = self.app.now()
-        assert app_now.tzinfo == tz_us_eastern
+
+        assert app_now.tzinfo.zone == tz_us_eastern.zone
 
         diff = to_utc(datetime.utcnow()) - localize(app_now, tz_utc)
         assert diff <= timedelta(seconds=1)
@@ -102,7 +101,7 @@ class test_App:
         del self.app.timezone
         app_now = self.app.now()
         assert self.app.timezone == tz_us_eastern
-        assert app_now.tzinfo == tz_us_eastern
+        assert app_now.tzinfo.zone == tz_us_eastern.zone
 
     @patch('celery.app.base.set_default_app')
     def test_set_default(self, set_default_app):
@@ -495,24 +494,6 @@ class test_App:
         i.annotate()
         i.annotate()
 
-    def test_apply_async_has__self__(self):
-        @self.app.task(__self__='hello', shared=False)
-        def aawsX(x, y):
-            pass
-
-        with pytest.raises(TypeError):
-            aawsX.apply_async(())
-        with pytest.raises(TypeError):
-            aawsX.apply_async((2,))
-
-        with patch('celery.app.amqp.AMQP.create_task_message') as create:
-            with patch('celery.app.amqp.AMQP.send_task_message') as send:
-                create.return_value = Mock(), Mock(), Mock(), Mock()
-                aawsX.apply_async((4, 5))
-                args = create.call_args[0][2]
-                assert args, ('hello', 4 == 5)
-                send.assert_called()
-
     def test_apply_async_adds_children(self):
         from celery._state import _task_stack
 
@@ -536,8 +517,8 @@ class test_App:
             _task_stack.pop()
 
     def test_pickle_app(self):
-        changes = dict(THE_FOO_BAR='bars',
-                       THE_MII_MAR='jars')
+        changes = {'THE_FOO_BAR': 'bars',
+                   'THE_MII_MAR': 'jars'}
         self.app.conf.update(changes)
         saved = pickle.dumps(self.app)
         assert len(saved) < 2048
@@ -786,6 +767,20 @@ class test_App:
         assert self.app.connection('amqp:////value') \
                        .failover_strategy == my_failover_strategy
 
+    def test_amqp_heartbeat_settings(self):
+        # Test default broker_heartbeat value
+        assert self.app.connection('amqp:////value') \
+                   .heartbeat == 0
+
+        # Test passing heartbeat through app configuration
+        self.app.conf.broker_heartbeat = 60
+        assert self.app.connection('amqp:////value') \
+                   .heartbeat == 60
+
+        # Test passing heartbeat as connection argument
+        assert self.app.connection('amqp:////value', heartbeat=30) \
+                   .heartbeat == 30
+
     def test_after_fork(self):
         self.app._pool = Mock()
         self.app.on_after_fork = Mock(name='on_after_fork')
@@ -826,17 +821,27 @@ class test_App:
 
     def test_timezone__none_set(self):
         self.app.conf.timezone = None
-        tz = self.app.timezone
-        assert tz == timezone.get_timezone('UTC')
+        self.app.conf.enable_utc = True
+        assert self.app.timezone == timezone.utc
+        del self.app.timezone
+        self.app.conf.enable_utc = False
+        assert self.app.timezone == timezone.local
 
     def test_uses_utc_timezone(self):
         self.app.conf.timezone = None
+        self.app.conf.enable_utc = True
         assert self.app.uses_utc_timezone() is True
 
+        self.app.conf.enable_utc = False
+        del self.app.timezone
+        assert self.app.uses_utc_timezone() is False
+
         self.app.conf.timezone = 'US/Eastern'
+        del self.app.timezone
         assert self.app.uses_utc_timezone() is False
 
         self.app.conf.timezone = 'UTC'
+        del self.app.timezone
         assert self.app.uses_utc_timezone() is True
 
     def test_compat_on_configure(self):
