@@ -2,13 +2,12 @@ from __future__ import absolute_import, unicode_literals
 
 import logging
 import os
-import pytest
 import sys
 import threading
 import warnings
-
 from importlib import import_module
 
+import pytest
 from case import Mock
 from case.utils import decorator
 from kombu import Queue
@@ -17,18 +16,17 @@ from celery.backends.cache import CacheBackend, DummyClient
 # we have to import the pytest plugin fixtures here,
 # in case user did not do the `python setup.py develop` yet,
 # that installs the pytest plugin into the setuptools registry.
-from celery.contrib.pytest import (
-    celery_app, celery_enable_logging, depends_on_current_app,
-)
-from celery.contrib.testing.app import Trap, TestApp
-from celery.contrib.testing.mocks import (
-    TaskMessage, TaskMessage1, task_message_from_sig,
-)
+from celery.contrib.pytest import (celery_app, celery_enable_logging,
+                                   celery_parameters, depends_on_current_app)
+from celery.contrib.testing.app import TestApp, Trap
+from celery.contrib.testing.mocks import (TaskMessage, TaskMessage1,
+                                          task_message_from_sig)
 
 # Tricks flake8 into silencing redefining fixtures warnings.
-__all__ = [
+__all__ = (
     'celery_app', 'celery_enable_logging', 'depends_on_current_app',
-]
+    'celery_parameters'
+)
 
 try:
     WindowsError = WindowsError  # noqa
