@@ -1,25 +1,17 @@
 from __future__ import absolute_import, unicode_literals
 
 import pickle
-
 from decimal import Decimal
+from itertools import count
 from random import shuffle
 from time import time
-from itertools import count
 
 from case import Mock, patch, skip
 
-from celery import states
-from celery import uuid
+from celery import states, uuid
 from celery.events import Event
-from celery.events.state import (
-    HEARTBEAT_EXPIRE_WINDOW,
-    HEARTBEAT_DRIFT_MAX,
-    State,
-    Worker,
-    Task,
-    heartbeat_expires,
-)
+from celery.events.state import (HEARTBEAT_DRIFT_MAX, HEARTBEAT_EXPIRE_WINDOW,
+                                 State, Task, Worker, heartbeat_expires)
 from celery.five import range
 
 
@@ -161,7 +153,7 @@ class ev_snapshot(replay):
             worker = not i % 2 and 'utest2' or 'utest1'
             type = not i % 2 and 'task2' or 'task1'
             self.events.append(Event('task-received', name=type,
-                               uuid=uuid(), hostname=worker))
+                                     uuid=uuid(), hostname=worker))
 
 
 class test_Worker:
