@@ -1,22 +1,15 @@
 from __future__ import absolute_import, unicode_literals
+
+import json
+
 import pytest
 from case import MagicMock, Mock
+
 from celery._state import _task_stack
-from celery.canvas import (
-    Signature,
-    chain,
-    _chain,
-    group,
-    chord,
-    signature,
-    xmap,
-    xstarmap,
-    chunks,
-    _maybe_group,
-    maybe_signature,
-    maybe_unroll_group,
-)
-from celery.result import AsyncResult, GroupResult, EagerResult
+from celery.canvas import (Signature, _chain, _maybe_group, chain, chord,
+                           chunks, group, maybe_signature, maybe_unroll_group,
+                           signature, xmap, xstarmap)
+from celery.result import AsyncResult, EagerResult, GroupResult
 
 SIG = Signature({
     'task': 'TASK',
@@ -187,6 +180,52 @@ class test_Signature(CanvasCase):
         s = signature('xxx.not.registered', app=self.app)
         assert s._apply_async
 
+    def test_keeping_link_error_on_chaining(self):
+        x = self.add.s(2, 2) | self.mul.s(4)
+        assert isinstance(x, _chain)
+        x.link_error(SIG)
+        assert SIG in x.options['link_error']
+
+        t = signature(SIG)
+        z = x | t
+        assert isinstance(z, _chain)
+        assert t in z.tasks
+        assert not z.options.get('link_error')
+        assert SIG in z.tasks[0].options['link_error']
+        assert not z.tasks[2].options.get('link_error')
+        assert SIG in x.options['link_error']
+        assert t not in x.tasks
+        assert not x.tasks[0].options.get('link_error')
+
+        z = t | x
+        assert isinstance(z, _chain)
+        assert t in z.tasks
+        assert not z.options.get('link_error')
+        assert SIG in z.tasks[1].options['link_error']
+        assert not z.tasks[0].options.get('link_error')
+        assert SIG in x.options['link_error']
+        assert t not in x.tasks
+        assert not x.tasks[0].options.get('link_error')
+
+        y = self.add.s(4, 4) | self.div.s(2)
+        assert isinstance(y, _chain)
+
+        z = x | y
+        assert isinstance(z, _chain)
+        assert not z.options.get('link_error')
+        assert SIG in z.tasks[0].options['link_error']
+        assert not z.tasks[2].options.get('link_error')
+        assert SIG in x.options['link_error']
+        assert not x.tasks[0].options.get('link_error')
+
+        z = y | x
+        assert isinstance(z, _chain)
+        assert not z.options.get('link_error')
+        assert SIG in z.tasks[3].options['link_error']
+        assert not z.tasks[1].options.get('link_error')
+        assert SIG in x.options['link_error']
+        assert not x.tasks[0].options.get('link_error')
+
 
 class test_xmap_xstarmap(CanvasCase):
 
@@ -260,6 +299,16 @@ class test_chain(CanvasCase):
     def test_from_dict_no_tasks(self):
         assert chain.from_dict(dict(chain(app=self.app)), app=self.app)
 
+    def test_from_dict_full_subtasks(self):
+        c = chain(self.add.si(1, 2), self.add.si(3, 4), self.add.si(5, 6))
+
+        serialized = json.loads(json.dumps(c))
+
+        deserialized = chain.from_dict(serialized)
+
+        for task in deserialized.tasks:
+            assert isinstance(task, Signature)
+
     @pytest.mark.usefixtures('depends_on_current_app')
     def test_app_falls_back_to_default(self):
         from celery._state import current_app
@@ -284,7 +333,7 @@ class test_chain(CanvasCase):
             self.add.s(30)
         )
         c._use_link = True
-        tasks, results = c.prepare_steps((), c.tasks)
+        tasks, results = c.prepare_steps((), {}, c.tasks)
 
         assert tasks[-1].args[0] == 5
         assert isinstance(tasks[-2], chord)
@@ -298,7 +347,7 @@ class test_chain(CanvasCase):
 
         c2 = self.add.s(2, 2) | group(self.add.s(i, i) for i in range(10))
         c2._use_link = True
-        tasks2, _ = c2.prepare_steps((), c2.tasks)
+        tasks2, _ = c2.prepare_steps((), {}, c2.tasks)
         assert isinstance(tasks2[0], group)
 
     def test_group_to_chord__protocol_2__or(self):
@@ -323,7 +372,7 @@ class test_chain(CanvasCase):
 
         c2 = self.add.s(2, 2) | group(self.add.s(i, i) for i in range(10))
         c2._use_link = False
-        tasks2, _ = c2.prepare_steps((), c2.tasks)
+        tasks2, _ = c2.prepare_steps((), {}, c2.tasks)
         assert isinstance(tasks2[0], group)
 
     def test_apply_options(self):
@@ -362,6 +411,26 @@ class test_chain(CanvasCase):
         self.app.conf.task_always_eager = True
         assert ~(self.add.s(4, 4) | self.add.s(8)) == 16
 
+    def test_chain_always_eager(self):
+        self.app.conf.task_always_eager = True
+        from celery import _state
+        from celery import result
+
+        fixture_task_join_will_block = _state.task_join_will_block
+        try:
+            _state.task_join_will_block = _state.orig_task_join_will_block
+            result.task_join_will_block = _state.orig_task_join_will_block
+
+            @self.app.task(shared=False)
+            def chain_add():
+                return (self.add.s(4, 4) | self.add.s(8)).apply_async()
+
+            r = chain_add.apply_async(throw=True).get()
+            assert r.get() == 16
+        finally:
+            _state.task_join_will_block = fixture_task_join_will_block
+            result.task_join_will_block = fixture_task_join_will_block
+
     def test_apply(self):
         x = chain(self.add.s(4, 4), self.add.s(8), self.add.s(10))
         res = x.apply()
@@ -372,6 +441,11 @@ class test_chain(CanvasCase):
         assert res.parent.parent.get() == 8
         assert res.parent.parent.parent is None
 
+    def test_single_expresion(self):
+        x = chain(self.add.s(1, 2)).apply()
+        assert x.get() == 3
+        assert x.parent is None
+
     def test_empty_chain_returns_none(self):
         assert chain(app=self.app)() is None
         assert chain(app=self.app).apply_async() is None
@@ -421,6 +495,16 @@ class test_chain(CanvasCase):
             seen.add(node.id)
             node = node.parent
 
+    def test_append_to_empty_chain(self):
+        x = chain()
+        x |= self.add.s(1, 1)
+        x |= self.add.s(1)
+        x.freeze()
+        tasks, _ = x._frozen
+        assert len(tasks) == 2
+
+        assert x.apply().get() == 3
+
 
 class test_group(CanvasCase):
 
@@ -536,6 +620,14 @@ class test_group(CanvasCase):
         g = group([self.add.s(i, i) for i in range(10)])
         assert list(iter(g)) == list(g.keys())
 
+    def test_single_task(self):
+        g = group([self.add.s(1, 1)])
+        assert isinstance(g, group)
+        assert len(g.tasks) == 1
+        g = group(self.add.s(1, 1))
+        assert isinstance(g, group)
+        assert len(g.tasks) == 1
+
     @staticmethod
     def helper_test_get_delay(result):
         import time
@@ -552,13 +644,15 @@ class test_group(CanvasCase):
 
     def test_kwargs_apply(self):
         x = group([self.add.s(), self.add.s()])
-        res = x.apply(kwargs=dict(x=1, y=1)).get()
+        res = x.apply(kwargs={'x': 1, 'y': 1}).get()
         assert res == [2, 2]
 
     def test_kwargs_apply_async(self):
         self.app.conf.task_always_eager = True
         x = group([self.add.s(), self.add.s()])
-        res = self.helper_test_get_delay(x.apply_async(kwargs=dict(x=1, y=1)))
+        res = self.helper_test_get_delay(
+            x.apply_async(kwargs={'x': 1, 'y': 1})
+        )
         assert res == [2, 2]
 
     def test_kwargs_delay(self):
@@ -607,6 +701,10 @@ class test_chord(CanvasCase):
         x = chord([t1], body=t2)
         assert x.app is t2._app
 
+    def test_app_when_header_is_empty(self):
+        x = chord([], self.add.s(4, 4))
+        assert x.app is self.add.app
+
     @pytest.mark.usefixtures('depends_on_current_app')
     def test_app_fallback_to_current(self):
         from celery._state import current_app
