@@ -1,15 +1,11 @@
 from __future__ import absolute_import, unicode_literals
+
 import pytest
 from case import Mock
+
 from celery.five import bytes_if_py2
-from celery.utils.imports import (
-    NotAPackage,
-    qualname,
-    gen_task_name,
-    reload_from_cwd,
-    module_file,
-    find_module,
-)
+from celery.utils.imports import (NotAPackage, find_module, gen_task_name,
+                                  module_file, qualname, reload_from_cwd)
 
 
 def test_find_module():
