@@ -1,6 +1,8 @@
 from __future__ import absolute_import, unicode_literals
+
 from case import skip
-from celery.utils.sysinfo import load_average, df
+
+from celery.utils.sysinfo import df, load_average
 
 
 @skip.unless_symbol('os.getloadavg')
